<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/
class MATCHMAKING_CLASS_PreferencesForm extends BASE_CLASS_UserQuestionForm
{
    private $questionService;

    public function __construct()
    {
        parent::__construct('MATCHMAKING_PreferencesForm');

        $this->questionService = BOL_QuestionService::getInstance();
        $language = OW::getLanguage();

        $save = new Submit('save');
        $save->setValue($language->text('matchmaking', 'btn_label_save'));
        $this->addElement($save);

    }

    public function process($questionArray, $data)
    {
        $language = OW::getLanguage();
        $user = OW::getUser()->getUserObject();
        
        $saveDistance = false;

        foreach ( $questionArray as $section )
        {
            foreach ( $section as $key => $question )
            {
                if ( $question['name'] == 'googlemap_location' )
                {
                    $saveDistance = true;
                }
                
                switch ( $question['presentation'] )
                {
                    case 'multicheckbox':

                        if ( is_array($data[$question['name']]) )
                        {
                            $data[$question['name']] = array_sum($data[$question['name']]);
                        }
                        else
                        {
                            $data[$question['name']] = 0;
                        }

                        break;
                }
            }
        }

        // save user data
        if ( !empty($user->id) )
        {
            if ( $this->questionService->saveQuestionsData($data, $user->id) )
            {
                if ( $saveDistance )
                {
                    MATCHMAKING_BOL_Service::getInstance()->saveDistance($user->id, (float)$data['distance_from_my_location']);
                }
                
                $event = new OW_Event(OW_EventManager::ON_USER_EDIT, array('userId' => $user->id, 'method' => 'native',));
                OW::getEventManager()->trigger($event);

                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    
    protected function getPresentationClass( $presentation, $questionName, $configs = null )
    {
        if ( $questionName == 'googlemap_location' )
        {
            $class = new MATCHMAKING_CLASS_DistanceField('distance_from_my_location');
            $class->addValidator(new GOOGLELOCATION_CLASS_DistanceValidator());
            $class->setLabel(OW::getLanguage()->text('matchmaking', 'distance_from_my_location_label'));
            
            return $class;
        }
        
        return BOL_QuestionService::getInstance()->getPresentationClass($presentation, $questionName, $configs);
    }
    
    protected function setLabel( $formField, $question )
    {
        $label = $formField->getLabel();

        if ( empty($label) )
        {
            $formField->setLabel(OW::getLanguage()->text('base', 'questions_question_' . $question['name'] . '_label'));
        }
        
        if ( $question['name'] == 'googlemap_location' )
        {
            $formField->setLabel(OW::getLanguage()->text('matchmaking', 'distance_from_my_location_label'));
        }
    }
    
    protected function setFieldValue( $formField, $presentation, $value )
    {
        $value = BOL_QuestionService::getInstance()->prepareFieldValue($presentation, $value);
        
        if ( $formField->getName() == 'distance_from_my_location' )
        {            
            $distanse = MATCHMAKING_BOL_Service::getInstance()->getDistance(OW::getUser()->getId());
            $formField->setValue($distanse);
        }
        else
        {
            $formField->setValue($value);
        }
    }
}