<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class WINKS_BOL_Service
{
    CONST EMAIL_SEND = 'send';
    CONST EMAIL_BACK = 'back';

    CONST LIMIT_TIMESTAMP = 604800; // week
    
    private static $classInstance;
    
    public static function getInstance()
    {
        if ( !isset(self::$classInstance) )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    private $winksDao;

    private function __construct()
    {
        $this->winksDao = WINKS_BOL_WinksDao::getInstance();
    }

    public function isLimited( $userId, $partnerId )
    {
        return $this->winksDao->isLimited($userId, $partnerId);
    }
    
    public function sendWink( $userId, $partnerId )
    {
        if ( empty($userId) || empty($partnerId) )
        {
            return FALSE;
        }

        if ( ($wink = $this->findByUserIdAndPartnerId($userId, $partnerId)) === NULL )
        {
            $wink = new WINKS_BOL_Winks();
        }
        
        $activeModes = json_decode(OW::getConfig()->getValue('mailbox', 'active_modes'));

        $wink->setUserId($userId);
        $wink->setPartnerId($partnerId);
        $wink->setTimeStamp(time());
        $wink->setStatus(WINKS_BOL_WinksDao::STATUS_WAIT);
        $wink->setViewed(0);
        $wink->setConversationId(0);
        $wink->setMessageType((in_array('chat', $activeModes) ? 'chat' : 'mail'));
        $wink->setWinkback(0);
        $this->winksDao->save($wink);

        return TRUE;
    }

    public function findByUserIdAndPartnerId( $userId, $partnerId )
    {
        return $this->winksDao->findByUserIdAndPartnerId($userId, $partnerId);
    }
    
    public function countWinksForUser( $partnerId, $status = NULL, $viewed = NULL, array $activeModes = array())
    {
        return $this->winksDao->countWinksForUser($partnerId, $status, $viewed, $activeModes);
    }
    
    public function countWinksForPartner( $userId, $status = NULL, $viewed = NULL )
    {
        return $this->winksDao->countWinksForPartner($userId, $status, $viewed);
    }
    
    public function countWinkBackedByUserId( $userId, array $activeModes = array() )
    {
        return $this->winksDao->countWinkBackedByUserId($userId, $activeModes);
    }

    public function findWinkList( $partnerId, $first, $limit, array $activeModes = array() )
    {
        return $this->winksDao->findWinkList($partnerId, $first, $limit, $activeModes);
    }

    public function findWinkListByStatus( $partnerId, $first, $limit, $status )
    {

        $winks = $this->winksDao->findWinkListByStatus($partnerId, $first, $limit, $status);

        if ( $winks )
        {
            $authors = array();

            // process winks
            foreach($winks as $wink)
            {
                if ( !in_array($wink->userId, $authors) )
                {
                    array_push($authors, $wink->userId);
                }
                
                $winkList[] = array(
                    'id' => $wink->id,
                    'userId' => $wink->userId,
                    'date' => UTIL_DateTime::formatDate($wink->timestamp),
                    'timestamp' => $wink->timestamp,
                    'viewed' => $wink->getViewed()
                );
            }

            $onlineUsers = BOL_UserService::getInstance()->findOnlineStatusForUserList($authors);
            $displayNames = BOL_UserService::getInstance()->getDisplayNamesForList($authors);
            $avatars = BOL_AvatarService::getInstance()->getDataForUserAvatars($authors, true, false, false, false);

            // complete users info
            foreach($winkList as &$wink)
            {
                $displayName = $displayNames[$wink['userId']];

                $wink['userOnline'] = !empty($onlineUsers[$wink['userId']]);
                $wink['text'] = strip_tags(OW::getLanguage()->text('winks', 'console_wink_wait_item', array('userUrl' => null, 'displayName' => $displayName)));
                $wink['avatar'] = $avatars[$wink['userId']]['src'];
                $wink['displayName'] = $displayName;
            }

            return $winkList;
        }

        return array();
    }

    public function markViewedByIds( $winksIds )
    {
        return $this->winksDao->markViewedByIds($winksIds);
    }
    
    public function findWinkByUserIdAndPartnerId( $userId, $partnerId )
    {
        return $this->winksDao->findWinkByUserIdAndPartnerId($userId, $partnerId);
    }
    
    public function findWinkById( $id )
    {
        return $this->winksDao->findById($id);
    }
    
    public function findExpiredDate( $timeStamp )
    {
        return $this->winksDao->findExpiredDate($timeStamp);
    }
    
    public function deleteWinkById( $id )
    {
        return $this->winksDao->deleteById($id);
    }
    
    public function deleteWinkByUserId( $userId )
    {
        return $this->winksDao->deleteWinkByUserId($userId);
    }
    
    public function setWinkback( $winkId, $flag = TRUE )
    {
        if ( empty($winkId) || ($wink = $this->winksDao->findById($winkId)) === NULL )
        {
            return FALSE;
        }
        
        $wink->setWinkback($flag);
        $this->winksDao->save($wink);
        
        return TRUE;
    }

    public function isWinkBacked( $winkId )
    {
        if ( empty($winkId) || ($wink = $this->findWinkById($winkId)) === null )
        {
            return false;
        }

        return $wink->getWinkback() == 1;
    }
    
    public function isCompleted( $userId, $partnerId )
    {
        return $this->winksDao->isCompleted($userId, $partnerId);
    }
    
    public function setStatusByUserId( $userId, $status )
    {
        return $this->winksDao->setStatusByUserId($userId, $status);
    }

    public function getActiveModes()
    {
        if ( OW::getPluginManager()->getPlugin('winks')->getDto()->getBuild() >= 14 )
        {
            return json_decode(OW::getConfig()->getValue('mailbox', 'active_modes'));
        }

         return array();
    }

    public function sendWinkEmailNotification( $userId, $partnerId, $winkType )
    {
        if ( empty($userId) || empty($partnerId) ||
            ($user = BOL_UserService::getInstance()->findUserById($userId)) === null ||
            ($partner = BOL_UserService::getInstance()->findUserById($partnerId)) === null )
        {
            return false;
        }

        $avatar = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($userId, $partnerId), true, true, true, false);

        switch ( $winkType )
        {
            case self::EMAIL_SEND:
                $subjectKey = 'wink_send_email_subject';
                $subjectArr = array('displayname' => $avatar[$userId]['title']);

                $textContentKey = 'wink_send_email_text_content';
                $htmlContentKey = 'wink_send_email_html_content';
                $contentArr = array(
                    'src' => $avatar[$userId]['src'],
                    'displayname' => $avatar[$userId]['title'],
                    'url' => $avatar[$userId]['url'],
                    'home_url' => OW_URL_HOME
                );
                break;
            case self::EMAIL_BACK:
            default:
                $subjectKey = 'wink_back_email_subject';
                $subjectArr = array('displayname' => $avatar[$userId]['title']);

                $textContentKey = 'wink_back_email_text_content';
                $htmlContentKey = 'wink_back_email_html_content';
                $contentArr = array(
                    'src' => $avatar[$userId]['src'],
                    'displayname' => $avatar[$userId]['title'],
                    'url' => $avatar[$userId]['url'],
                    'conversation_url' => OW::getRouter()->urlForRoute('mailbox_messages_default')
                );
                break;
        }

        $language = OW::getLanguage();
        $mail = OW::getMailer()->createMail();

        $mail->addRecipientEmail($partner->email);
        $mail->setSubject($language->text('winks', $subjectKey, $subjectArr));
        $mail->setTextContent($language->text('winks', $textContentKey, $contentArr));
        $mail->setHtmlContent($language->text('winks', $htmlContentKey, $contentArr));

        try
        {
            OW::getMailer()->send($mail);
        }
        catch ( Exception $e )
        {
            OW::getLogger('winks.send_notification')->addEntry(json_encode($e));
        }
    }
}
